// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _path = _interopRequireDefault(require("path"));
var _structuredSource = require("structured-source");
var _eslint = require("eslint");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
var defaultOptions = {
  // path to .eslintrc file
  "configFile": null,
  // recognize lang of CodeBlock
  "langs": ["js", "javascript", "node", "jsx"]
};
var getConfigBaseDir = context => {
  if (typeof context.getConfigBaseDir === "function") {
    return context.getConfigBaseDir();
  }
  // Fallback that use deprecated `config` value
  // https://github.com/textlint/textlint/issues/294
  var textlintRcFilePath = context.config ? context.config.configFile : null;
  // .textlinrc directory
  return textlintRcFilePath ? _path.default.dirname(textlintRcFilePath) : process.cwd();
};
var reporter = (context, options) => {
  var {
    Syntax,
    RuleError,
    report,
    fixer,
    getSource,
    getFilePath
  } = context;
  if (!options) {
    throw new Error("Require options: { \"configFile\": \"path/to/.eslintrc\" }");
  }
  if (!options.configFile) {
    throw new Error("Require options: { \"configFile\": \"path/to/.eslintrc\" }");
  }
  var availableLang = options.langs || defaultOptions.langs;
  var textlintRCDir = getConfigBaseDir(context);
  var esLintConfigFilePath = textlintRCDir ? _path.default.resolve(textlintRCDir, options.configFile) : options.configFile;
  var engine = new _eslint.ESLint({
    useEslintrc: false,
    overrideConfigFile: esLintConfigFilePath
  });
  return {
    [Syntax.CodeBlock](node) {
      return _asyncToGenerator(function* () {
        if (availableLang.indexOf(node.lang) === -1) {
          return;
        }
        var raw = getSource(node);
        var code = getUntrimmedCode(node, raw);
        var source = new _structuredSource.StructuredSource(code);
        var resultLinting = yield engine.lintText(code, {
          filePath: "test.".concat(node.lang)
        });
        if (resultLinting.length === 0) {
          return;
        }
        resultLinting.forEach(result => {
          result.messages.forEach(message => {
            /*
              1. ```js
             2. CODE
             3. ```
              ESLint message line and column start with 1
             */
            if (options.ignoreParsingErrors && message.message.includes("Parsing error")) {
              return;
            }
            var prefix = message.ruleId ? "".concat(message.ruleId, ": ") : "";
            if (message.fix) {
              var fixedRange = message.fix.range;
              var fixedText = message.fix.text;
              var sourceBlockDiffIndex = raw !== node.value ? raw.indexOf(code) : 0;
              var fixedWithPadding = [fixedRange[0] + sourceBlockDiffIndex, fixedRange[1] + sourceBlockDiffIndex];
              var index = source.positionToIndex({
                line: message.line,
                column: message.column
              });
              var adjustedIndex = index + sourceBlockDiffIndex - 1;
              report(node, new RuleError("".concat(prefix).concat(message.message), {
                index: adjustedIndex,
                fix: fixer.replaceTextRange(fixedWithPadding, fixedText)
              }));
            } else {
              var _sourceBlockDiffIndex = raw !== node.value ? raw.indexOf(code) : 0;
              var _index = source.positionToIndex({
                line: message.line,
                column: message.column
              });
              var _adjustedIndex = _index + _sourceBlockDiffIndex - 1;
              report(node, new RuleError("".concat(prefix).concat(message.message), {
                index: _adjustedIndex
              }));
            }
          });
        });
      })();
    }
  };
};

/**
 * [Markdown] get actual code value from CodeBlock node
 * @param {Object} node
 * @param {string} raw raw value include CodeBlock syntax
 * @returns {string}
 */
function getUntrimmedCode(node, raw) {
  if (node.type !== "CodeBlock") {
    return node.value;
  }
  // Space indented CodeBlock that has not lang
  if (!node.lang) {
    return node.value;
  }

  // If it is not markdown codeBlock, just use node.value
  if (!(raw.startsWith("```") && raw.endsWith("```"))) {
    if (node.value.endsWith("\n")) {
      return node.value;
    }
    return node.value + "\n";
  }
  // Markdown(remark) specific hack
  // https://github.com/wooorm/remark/issues/207#issuecomment-244620590
  var lines = raw.split("\n");
  // code lines without the first line and the last line
  var codeLines = lines.slice(1, lines.length - 1);
  // add last new line
  // \n```
  return codeLines.join("\n") + "\n";
}
var _default = {
  linter: reporter,
  fixer: reporter
};
exports.default = _default;
//# sourceMappingURL=textlint-rule-eslint.js.map